#========================================================================================
# package      DE_EPAGES::Ipayment::t::ChangeActionTrigger_HiddenTriggerOrderBeforePayment_si
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration test to change action hidden trigger
#========================================================================================
package DE_EPAGES::Ipayment::t::ChangeActionTrigger_HiddenTriggerOrderBeforePayment_si;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath ResetFactoryCaches );
use LWP::UserAgent;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->testHiddenTriggerPositivRequest();
    done_testing();
}

sub testHiddenTriggerPositivRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $OrderPath = $self->shopPath().'/Customers/TestCustomer/Orders/1234-5678';
    my $CustomerOrder = LoadObjectByPath($OrderPath);
    my $DataBaseBasketOrOrder = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    ok ($DataBaseBasketOrOrder->extractFromBasketOrOrder($CustomerOrder),"$SubTestName - tests if extractFromBasketOrOrder works fine");
    my $HiddenTriggerURL = $DataBaseBasketOrOrder->getHiddenTriggerUrl();    note('Used HiddenTriggerUrl: '.$HiddenTriggerURL);
    my $LineItemPaymentIpayment = $CustomerOrder->container()->get('Payment');
    my $LineItemPaymentIpaymentID = $LineItemPaymentIpayment->get('ID');

    if(isValid($LineItemPaymentIpayment->get('TransStatus')) ||
        isValid($LineItemPaymentIpayment->get('TransID'))){
            Error('LineItemPayment TransStatus or TransID are not undef!!!');
    }
    my $UserAgent = LWP::UserAgent->new();

    my $Response = $UserAgent->post($HiddenTriggerURL, _CreateResponseHash());

    my $ExpectedHttpStatusCode = 200; ## no critic (ProhibitMagicNumbers)
    is($Response->{'_rc'}, $ExpectedHttpStatusCode, "$SubTestName - Test if we get a right response - 200 ok");

    ResetFactoryCaches(); # reset Database cache since this Test-perl process is not the same as the appserver process

    my $LineItemPaymentIpaymentReload = LoadObject($LineItemPaymentIpaymentID);
    is($LineItemPaymentIpaymentReload->get('TransStatus'),
        'STATUS_ACCEPTED',
        "$SubTestName - test if the hidden trigger change action set the lineitempaymentipayment to accepted"
    );
    is($LineItemPaymentIpaymentReload->get('TransID'),
        '1-101971090',
        "$SubTestName -test if the hidden trigger change action set in the lineitempaymentipayment the correct transaction number"
    );
    return;
}



sub _CreateResponseHash(){
    my $ReturnResponseHash = {
        'shopper_id' => '53AC1BFE-5362-E914-FC55-AC1532072208',
        'REMOTE_ADDR'  => '212.227.34.218',
        'ret_trx_number'  => '1-101971090',
        'ret_status'  => 'SUCCESS',
        'trx_amount'  => '1000',
        'trx_paymenttyp'  => 'cc',
        'trx_currency'  => 'EUR',
        'trxuser_id'  => '99999',
        'ret_errorcode'  => '0',
        'trx_typ'  => 'preauth',
    };
    return $ReturnResponseHash;
}


#----------------------------------------------------------------------------------------
# Methods inherited from DE_EPAGES::Shop::API::Test
#----------------------------------------------------------------------------------------

sub shopAlias {
    return 'TestShopHiddenTriggerIpayment';
}
sub importXMLPath {
    return 'ShopXMLs/ImportShopChangeActionHiddenTrigger.xml';
}

__PACKAGE__->runScript();

1;
