#========================================================================================
# package      DE_EPAGES::Ipayment::t::ChangeActionTrigger_PaymentRedirectIpayment_si
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  system integration test for PaymentRedirectIpayment change action
#========================================================================================
package DE_EPAGES::Ipayment::t::ChangeActionTrigger_PaymentRedirectIpayment_si;
use base DE_EPAGES::Shop::API::TestImportXML;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use Test::More;

use strict;

#----------------------------------------------------------------------------------------
# Wrap for DE_EPAGES::Shop::API::TestImportXML
# You need to override this methods in other to use
#----------------------------------------------------------------------------------------
sub shopAlias {
    return 'TestShopChangeActionIpayment';
}
sub importXMLPath {
    return 'ShopXMLs/ChangeActionTrigger_PaymentRedirectIpayment.xml';
}
sub cartridgeName {
    return 'DE_EPAGES::Ipayment';
}

#----------------------------------------------------------------------------------------
sub test {
    my $self = shift;
    $self->test_PaymentRedirectIpayment_Positive();
    $self->test_PaymentRedirectIpayment_Negative_noShopperID();
    $self->test_PaymentRedirectIpayment_Negative_ShopperIDFromNotLineItemPayment();
    $self->test_PaymentRedirectIpayment_Negative_WrongShopperID();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_PaymentRedirectIpayment_Positive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $RedirectURL = BuildShopUrl(
        $self->shop(),
        {'ChangeAction' => 'Redirect'},
        {'Type' => 'sf', 'UseSSL' => 1,}
     );

    note('Used RedirectURL: '.$RedirectURL);

    my $UserAgent = LWP::UserAgent->new();

    my $LineItemPaymentGUID = '53AC1BFE-5362-E914-FC55-AC1532072208';
    my $Response = $UserAgent->post($RedirectURL, {'shopper_id' => $LineItemPaymentGUID});
    my $ExpectedHttpStatusCode = 200; ## no critic (ProhibitMagicNumbers)
    is($Response->{'_rc'}, $ExpectedHttpStatusCode, "$SubTestName - Test if we get a right response - 200 ok");
    return;
}

#----------------------------------------------------------------------------------------
sub test_PaymentRedirectIpayment_Negative_noShopperID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $RedirectURL = BuildShopUrl(
        $self->shop(),
        {'ChangeAction' => 'Redirect'},
        {'Type' => 'sf', 'UseSSL' => 1,}
     );

    note('Used RedirectURL: '.$RedirectURL);

    my $UserAgent = LWP::UserAgent->new();

    my $EmptyLineItemPaymentGUID = {};
    my $Response = $UserAgent->post($RedirectURL, $EmptyLineItemPaymentGUID);
    my $ExpectedHttpStatusCode = 200; ## no critic (ProhibitMagicNumbers)
    is($Response->{'_rc'}, $ExpectedHttpStatusCode, "$SubTestName - Test if we get a right response - 200");
    return;
}

#----------------------------------------------------------------------------------------
sub test_PaymentRedirectIpayment_Negative_ShopperIDFromNotLineItemPayment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $RedirectURL = BuildShopUrl(
        $self->shop(),
        {'ChangeAction' => 'Redirect'},
        {'Type' => 'sf', 'UseSSL' => 1,}
     );

    note('Used RedirectURL: '.$RedirectURL);

    my $UserAgent = LWP::UserAgent->new();

    my $NotLineItemPaymentGUID = $self->shop()->guid();
    my $Response = $UserAgent->post($RedirectURL, {'shopper_id' => $NotLineItemPaymentGUID});
    my $ExpectedHttpStatusCode = 200; ## no critic (ProhibitMagicNumbers)
    is($Response->{'_rc'}, $ExpectedHttpStatusCode, "$SubTestName - Test if we get a right response - 200");
    return;
}

#----------------------------------------------------------------------------------------
sub test_PaymentRedirectIpayment_Negative_WrongShopperID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $RedirectURL = BuildShopUrl(
        $self->shop(),
        {'ChangeAction' => 'Redirect'},
        {'Type' => 'sf', 'UseSSL' => 1,}
     );

    note('Used RedirectURL: '.$RedirectURL);

    my $UserAgent = LWP::UserAgent->new();

    my $NotLineItemPaymentGUID = $self->shop()->guid();
    my $Response = $UserAgent->post($RedirectURL, {'shopper_id' => $NotLineItemPaymentGUID});
    my $ExpectedHttpStatusCode = 200; ## no critic (ProhibitMagicNumbers)
    is($Response->{'_rc'}, $ExpectedHttpStatusCode, "$SubTestName - Test if we get a right response - 200");
    return;
}


__PACKAGE__->runScript();

1;