#========================================================================================
# package      DE_EPAGES::Ipayment::t::PaymentTypeMethodsGroups
# state        public
#----------------------------------------------------------------------------------------
# description  PaymentTypeMethods
#========================================================================================
package DE_EPAGES::Ipayment::t::PaymentTypeMethodsGroups;
use base DE_EPAGES::Shop::API::Test;

use strict;

use Test::More;
use DE_EPAGES::ExternalPayment::API::PaymentTypeMethods qw ( GetPaymentTypeMethodsGroups );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
);


my $SampleData = {
    'PaymentTypePath' => '/PaymentTypes/Ipayment',
    'LanguageID' => 2,
    'PaymentTypeMethodsGroupsCount' => 4,
    'PaymentTypeMethodsGroupsName' => 'Credit Cards',
    'PaymentTypeMethodsCount' => 9,
    'PaymentTypeMethodsName' => 'Master-Card',
} ;

#----------------------------------------------------------------------------------------

sub test {
    my $self = shift;

    my $PaymentTypePath = $SampleData->{'PaymentTypePath'} ;
    if ( ExistsObjectByPath( $PaymentTypePath ) ) {
        TestPaymentType( LoadObjectByPath( $PaymentTypePath ) ) ;
    } else {
        fail( "'$PaymentTypePath' NOT Object defined." ) ;
    }

    done_testing();
}
#----------------------------------------------------------------------------------------
sub TestPaymentType {
    my ( $PaymentType ) = @_ ;

    ok( 1, 'PaymentType Object defined.' ) ;
    my $PaymentTypeMethodsGroups = GetPaymentTypeMethodsGroups( $PaymentType, $SampleData->{'LanguageID'} ) ;
    TestPaymentTypeMethodsGroups( $PaymentTypeMethodsGroups ) ;

    return ;
}

sub TestPaymentTypeMethodsGroups {
    my ( $PaymentTypeMethodsGroups ) = @_ ;

    my $PaymentTypeMethodsGroupsCount = $SampleData->{'PaymentTypeMethodsGroupsCount'} ;
    is(
        scalar @$PaymentTypeMethodsGroups,
        $PaymentTypeMethodsGroupsCount,
        "PaymentTypeMethodsGroups expected : $PaymentTypeMethodsGroupsCount"
    ) ;

    my $PaymentTypeMethodsGroupsName = $SampleData->{'PaymentTypeMethodsGroupsName'} ;
    foreach my $PaymentTypeMethodsGroup ( @{$PaymentTypeMethodsGroups} ) {
        if ( $PaymentTypeMethodsGroup->{'Name'} eq $PaymentTypeMethodsGroupsName ) {
            ok( 1, "Found Sample Group '$PaymentTypeMethodsGroupsName'" ) ;
            TestPaymentTypeMethods( $PaymentTypeMethodsGroup->{'PaymentTypeMethods'} ) ;
            return ;
        }
    }
    ok( 0, "Sample Group '$PaymentTypeMethodsGroupsName' Not Found !!!" ) ;

    return ;
}

sub TestPaymentTypeMethods {
    my ( $PaymentTypeMethods ) = @_ ;

    my $PaymentTypeMethodsCount = $SampleData->{'PaymentTypeMethodsCount'} ;
    is(
        scalar @$PaymentTypeMethods,
        $PaymentTypeMethodsCount,
        "PaymentTypeMethods expected : $PaymentTypeMethodsCount"
    ) ;

    my $PaymentTypeMethodsName = $SampleData->{'PaymentTypeMethodsName'} ;
    foreach my $PaymentTypeMethods ( @$PaymentTypeMethods ) {
        if ( $PaymentTypeMethods->{'Name'} eq $PaymentTypeMethodsName ) {
            ok( 1, "Found Sample Type '$PaymentTypeMethodsName'" ) ;
            return ;
        }
    }
    ok( 0, "Sample Type '$PaymentTypeMethodsName' Not Found !!!" ) ;

    return ;
}


__PACKAGE__->runScript();

1;