#========================================================================================
# package      DE_EPAGES::Ipayment::t::SOAP_CreateSession_Negative_si
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration Test to create session
#========================================================================================
package DE_EPAGES::Ipayment::t::SOAP_CreateSession_Negative_si;
use base DE_EPAGES::Shop::API::TestImportXML;

use strict;

use Readonly;

use DE_EPAGES::Ipayment::API::Session;
use DE_EPAGES::Ipayment::t::unit::TestTools qw ( ImportTestXmlFile CreatePositiveXMLHttpResponse CreateMockedUserAgent IsXml Xml2Hash CreateCompiledClientWithCustomUserAgent );

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Core::API::File qw ( GetFileContent );

use Test::More;
use Test::MockObject::Extends;


#----------------------------------------------------------------------------------------
# Wrap for DE_EPAGES::Shop::API::TestImportXML
# You need to override this methods in other to use
#----------------------------------------------------------------------------------------
sub shopAlias {
    return 'SoapCreateSessionNegativeShop';
}
sub importXMLPath {
    return 'ShopXMLs/importNegative.xml';
}

#----------------------------------------------------------------------------------------

sub test {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    Readonly my $ORDERPATH => $self->shopPath().'/Customers/TestCustomer/Orders/1234-5678';
    my $Order;

   #prepare
    $Order = LoadObjectByPath($ORDERPATH);
    my $Session = DE_EPAGES::Ipayment::API::Session->new();
    my $Status;
    my $MockedLWPUserAgent = undef;
    if ($ENV{'SKIP_EXTERNAL_TESTS'} == 1) {
        note('mock request to Ipayment');
        my $FileName = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Ipayment/t/TestResponses/CreateSessionNegativeResponse.xml"; ## no critic (ProhibitLongLines)
        my $Content = ImportTestXmlFile($FileName);
        my $Response = CreatePositiveXMLHttpResponse($Content);
        $MockedLWPUserAgent = CreateMockedUserAgent($Response);
        $Session = OverrwriteUserAgentInSession($Session,$MockedLWPUserAgent);
    }

    #request
    $Status = $Session->createSessionAtIpaymentAndStoreIdInDatabase($Order);

    #test
    if ($ENV{'SKIP_EXTERNAL_TESTS'} == 1) {
        my $RequestXml = $MockedLWPUserAgent->{'Request_MockVar'};
        ok(IsXml($RequestXml), "$SubTestName - test if request is a valid xml");
        my $RequestXmlHash = Xml2Hash($RequestXml);
        my $RequestXmlHashExpected = Xml2Hash(_GetRequest());
        ## no critic (ProhibitLongLines)
        is($RequestXmlHash->{'ns1:createSession'}->{'accountData'}->{'accountId'}, $RequestXmlHashExpected->{'ns1:createSession'}->{'accountData'}->{'accountId'}, "$SubTestName - test if accountId is not correct");
        is($RequestXmlHash->{'ns1:createSession'}->{'accountData'}->{'trxpassword'}, $RequestXmlHashExpected->{'ns1:createSession'}->{'accountData'}->{'trxpassword'}, "$SubTestName - test if trxpassword is not correct");
        is($RequestXmlHash->{'ns1:createSession'}->{'accountData'}->{'trxuserId'}, $RequestXmlHashExpected->{'ns1:createSession'}->{'accountData'}->{'trxuserId'}, "$SubTestName - test if trxuserId is not correct");
        is($RequestXmlHash->{'ns1:createSession'}->{'accountData'}->{'adminactionpassword'}, $RequestXmlHashExpected->{'ns1:createSession'}->{'accountData'}->{'adminactionpassword'}, "$SubTestName - test if adminactionpassword is not correct");
        ## use critic (ProhibitLongLines)
    }
    ok(!$Status, "$SubTestName - tests if the session has not be created");
    my $SessionID = $Order->container()->get('Payment')->get('SessionID');
    ok(!isValid($SessionID), "$SubTestName - tests if the session is not stored in the database");

    done_testing();
}
#----------------------------------------------------------------------------------------
sub OverrwriteUserAgentInSession {

    my ($Session,$MockedLWPUserAgent) = @_;
    $Session = Test::MockObject::Extends->new( $Session );
    $Session->mock('_createCompiledClient', sub {
        my $self = shift;
        my ($EndpointURL, $OperationName) = @_;
        return CreateCompiledClientWithCustomUserAgent($EndpointURL , $OperationName, $MockedLWPUserAgent);
     });
}



sub _GetRequest {
    my $TestResponsePath = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Ipayment/t/TestRequests/";
    return ${GetFileContent($TestResponsePath.'CreateSessionRequest.xml')};
}


__PACKAGE__->runScript();

1;
