#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::CGI_SecurityLayer
# state        public
#----------------------------------------------------------------------------------------
# description  Test package for DE_EPAGES::Ipayment::API::CGI::SecurityLayer
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::CGI_SecurityLayer;

use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Ipayment::API::CGI::SecurityLayer;
use DE_EPAGES::Ipayment::API::Object::LineItemPaymentIpayment;
use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;
use DE_EPAGES::Ipayment::t::unit::TestTools qw ( TestExistsMethod );
use DE_EPAGES::Order::API::Object::LineItemContainer;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::MockObject;
use Test::MockObject::Extends;
use Test::More;

use strict;

#----------------------------------------------------------------------------------------
sub test {
    my $self = shift;
    $self->test_CallWithInvalidParametersAndValidShopperId();
    $self->test_CallWithInvalidParametersAndInvalidShopperId();
    $self->test_CallWithValidParameters();
    $self->test_UncompletedParamsList();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_CallWithInvalidParametersAndValidShopperId {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $DatabaseBasketOrOrder = $self->_createDatabaseBasketOrOrder(1);
    my $SecurityLayer = $self->_createSecurityLayer($DatabaseBasketOrOrder);
    my $hParams = $self->_createInvalidParams();
    $hParams->{'shopper_id'} = '53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2';
    my $IsValidParamerters = $SecurityLayer->extractDataFromParams($hParams);
    is($IsValidParamerters, 0, "$SubTestName - checks the behavior without a parameter");
    is($SecurityLayer->getShopperID(), '53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2', "$SubTestName - checks the getter of ShopperID");
    is($SecurityLayer->getLineItemPayment()->alias(),
        'LineItemPaymentIpayment_53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2',
        "$SubTestName - checks if the returning line item payment has the right alias"
    );
    my $ValidShopID = 123456; ## no critic qw( ProhibitMagicNumbers )
    ok($SecurityLayer->belongsThisOrdertoShop($ValidShopID), "$SubTestName - checks if an order belongs to the shop");
    is($SecurityLayer->getReturnStatus(), undef, "$SubTestName - checks the return status");
    is($SecurityLayer->getTransactionNumber(), undef, "$SubTestName - checks the return status");
    is($SecurityLayer->getTransactionType(), undef, "$SubTestName - checks the return status");
    is($SecurityLayer->getRemoteIp(), '212.227.34.666', "$SubTestName - checks the return remote ip");
    is($SecurityLayer->getErrorMessage(), 'RemoteIpError', "$SubTestName - checks the SecurityLayer Error message");
    is(@{$SecurityLayer->{'params_MockVar'}}[0], 'LineItemPaymentIpayment',  "$SubTestName - checks parameters of mocked _LoadClassByAlias method");
    is(@{$SecurityLayer->{'GetObjectParams_MockVar'}}[0], 'ByGUID', "$SubTestName - checks parameters of mocked getObjects method");
    is_deeply(@{$SecurityLayer->{'GetObjectParams_MockVar'}}[1], ['53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2'], "$SubTestName - checks parameters of mocked getObjects method");
    is($SecurityLayer->getTransactionAmount(), '1245', "$SubTestName - checks if we get a value if the trx_amount is  set");
}

#----------------------------------------------------------------------------------------
sub test_CallWithInvalidParametersAndInvalidShopperId {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $DatabaseBasketOrOrder = $self->_createDatabaseBasketOrOrder(1);
    my $SecurityLayer = $self->_createSecurityLayer($DatabaseBasketOrOrder);
    my $hParams = $self->_createInvalidParams();
    $hParams->{'shopper_id'} = '';
    my $IsValidParamerters = $SecurityLayer->extractDataFromParams($hParams);
    is($IsValidParamerters, 0, "$SubTestName - checks the behavior without a parameter");
    is($SecurityLayer->getShopperID() , undef, "$SubTestName - checks the getter of ShopperID");
    throws_ok { $SecurityLayer->getLineItemPayment() } (
       qr/Invalid TransactionID: /,
       "$SubTestName - Calling the getLineItemPayment is not possible"
    );
    throws_ok { $SecurityLayer->belongsThisOrdertoShop() } (
       qr/Invalid TransactionID: /,
       "$SubTestName - Calling the belongsThisOrdertoShop is not possible"
    );
    is($SecurityLayer->getReturnStatus(), undef, "$SubTestName - checks the return status");
    is($SecurityLayer->getTransactionNumber(), undef, "$SubTestName - checks the return status");
    is($SecurityLayer->getTransactionType(), undef, "$SubTestName - checks the return status");
    is($SecurityLayer->getRemoteIp(), undef, "$SubTestName - checks the return remote ip");
    is_deeply($SecurityLayer->getErrorMessage(), 'Form Parameters have no shopper id', "$SubTestName - checks the SecurityLayer Error message");
    is(@{$SecurityLayer->{'params_MockVar'}}[0],'LineItemPaymentIpayment', "$SubTestName - checks parameters of mocked _LoadClassByAlias method");
    is(@{$SecurityLayer->{'GetObjectParams_MockVar'}}[0], 'ByGUID', "$SubTestName - checks parameters of mocked getObjects method");
    is_deeply (@{$SecurityLayer->{'GetObjectParams_MockVar'}}[1], [undef], "$SubTestName - checks parameters of mocked getObjects method");
    is($SecurityLayer->getTransactionAmount(), undef, "$SubTestName - checks if we get a value if the trx_amount is  set");
}

#----------------------------------------------------------------------------------------
sub test_CallWithValidParameters {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $DatabaseBasketOrOrder = $self->_createDatabaseBasketOrOrder(1);
    my $SecurityLayer = $self->_createSecurityLayer($DatabaseBasketOrOrder);
    my $hParams = $self->_createParams();

    my $IsValidParamerters = $SecurityLayer->extractDataFromParams($hParams);
    is($IsValidParamerters, 1, "$SubTestName - checks the behavior with parameters");
    is($SecurityLayer->getShopperID() , '53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2', "$SubTestName - checks the getter of ShopperID");
    is($SecurityLayer->getLineItemPayment()->alias(),
        'LineItemPaymentIpayment_53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2',
        "$SubTestName - checks if the returning line item payment has the right alias"
    );
    is(@{$SecurityLayer->{'params_MockVar'}}[0], 'LineItemPaymentIpayment',  "$SubTestName - checks parameters of mocked _LoadClassByAlias method");
    is(@{$SecurityLayer->{'GetObjectParams_MockVar'}}[0], 'ByGUID', "$SubTestName - checks parameters of mocked getObjects method");
    is_deeply (@{$SecurityLayer->{'GetObjectParams_MockVar'}}[1], ['53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2'], "$SubTestName - checks parameters of mocked getObjects method");
    my $ValidShopID = 123456; ## no critic qw( ProhibitMagicNumbers )
    ok ($SecurityLayer->belongsThisOrdertoShop($ValidShopID), "$SubTestName - checks if an order belongs to the shop");
    my $InvalidShopID = 666; ## no critic qw( ProhibitMagicNumbers )
    is ($SecurityLayer->belongsThisOrdertoShop($InvalidShopID), 0, "$SubTestName - checks if an invalid order not belongs to the shop");
    is($SecurityLayer->getReturnStatus(), undef, "$SubTestName - checks the return status");
    is($SecurityLayer->getTransactionNumber(), undef, "$SubTestName - checks the return status");
    is($SecurityLayer->getTransactionType(), 'auth', "$SubTestName - checks the return status");
    is($SecurityLayer->getRemoteIp(), '212.227.34.219', "$SubTestName - checks the return remote ip");
    is_deeply($SecurityLayer->getErrorMessage(), undef, "$SubTestName - checks the SecurityLayer Error message");
    is($SecurityLayer->getTransactionAmount(), '1245', "$SubTestName - checks if we get a value if the trx_amount is  set");
}

#----------------------------------------------------------------------------------------
sub test_UncompletedParamsList {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $DatabaseBasketOrOrder = $self->_createDatabaseBasketOrOrder(1);
    my $SecurityLayer = $self->_createSecurityLayer($DatabaseBasketOrOrder);
    my $hParams = {
        'SomeKey' => 'bla',
        'trx_amount' =>  '1245'
    };
    is($SecurityLayer->extractDataFromParams($hParams), 0, "$SubTestName - checks the return value if the ShopperID Key is missing");
    is($SecurityLayer->getShopperID(), undef, "$SubTestName - checks if we get an undef if the shopper_id is not set");
    is($SecurityLayer->getTransactionAmount(), undef, "$SubTestName - checks if we get a value if the trx_amount is  set");
}

#----------------------------------------------------------------------------------------
sub _createDatabaseBasketOrOrder {
    my $self = shift;
    my ($ExtractionWasSuccessful) = @_;

    my $DatabaseBasketOrOrder = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    my $MockedDatabaseBasketOrOrder = Test::MockObject::Extends->new($DatabaseBasketOrOrder);
    $MockedDatabaseBasketOrOrder->{'PaymentTypeMethod'} = $self->_createPaymentTypeMethod();
    $MockedDatabaseBasketOrOrder->{'LineItemContainer'} = $self->_createLineItemContainer();
    $MockedDatabaseBasketOrOrder->{'LineItemPayment'} = $self->_createLineItemPaymentIpayment();

    $MockedDatabaseBasketOrOrder->mock('extractFromBasketOrOrder', sub{
        shift;
        my ($Order) = @_;
            return $ExtractionWasSuccessful;
    });
    $MockedDatabaseBasketOrOrder->mock('getGrandTotal', sub{
        shift;
        return 12.45; ## no critic qw( ProhibitMagicNumbers )
    });
    $MockedDatabaseBasketOrOrder->mock('getGUID', sub{
        shift;
        return '53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2';
    });
    $MockedDatabaseBasketOrOrder->mock('getCurrencyID', sub{
        shift;
        return 'GBP';
    });
    $MockedDatabaseBasketOrOrder->mock('getUserID', sub{
        shift;
        return '99999';
    });
    $MockedDatabaseBasketOrOrder->mock('container', sub{
        shift;
        return $MockedDatabaseBasketOrOrder->{'LineItemContainer'};
    });
    return $MockedDatabaseBasketOrOrder;
}

#----------------------------------------------------------------------------------------
sub _createSecurityLayer {
    my $self = shift;
    my ($DatabaseBasketOrOrder) = @_;

    my $SecurityLayer = DE_EPAGES::Ipayment::API::CGI::SecurityLayer->new($DatabaseBasketOrOrder);
    my $MockedSecurityLayer =  Test::MockObject::Extends->new($SecurityLayer);

    TestExistsMethod($SecurityLayer, '_LoadClassByAlias');
    $MockedSecurityLayer->mock('_LoadClassByAlias', sub{
        my $MockedSecurityLayer_self = shift;
        ($MockedSecurityLayer_self->{'params_MockVar'}, $MockedSecurityLayer_self->{'LineItem_MockVar'}) = \@_;

        my $LineItemPaymentIpaymentClass = Test::MockObject->new();
        $LineItemPaymentIpaymentClass->mock('getObjects', sub{
            shift;
            $MockedSecurityLayer_self->{'GetObjectParams_MockVar'} = \@_;

            if (isValid($MockedSecurityLayer_self->{'GetObjectParams_MockVar'}[1][0])) {
                return [$self->{'LineItemPayment'}];
            } else {
                return undef;
            }
        });
        return $LineItemPaymentIpaymentClass;
    });
    return $MockedSecurityLayer;
}

#----------------------------------------------------------------------------------------
sub _createLineItemPaymentIpayment{
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $Attributes = {
        'GUID'      => '53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2',
        'Parent'    => $self->{'LineItemContainer'},
        'Alias'     => 'LineItemPaymentIpayment_53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2',
        'Site'      => $self->_createSite(),
    };
    my $MockedLineItemPayment = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Ipayment::API::Object::LineItemPaymentIpayment', $Attributes);
    $self->{'LineItemPayment'} = $MockedLineItemPayment;
    return $MockedLineItemPayment;
}

#----------------------------------------------------------------------------------------
sub _createSite {
    my $self = shift;

    my $Attributes = {
        'ID' => 123456, ## no critic qw( ProhibitMagicNumbers )
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', $Attributes);
}

#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;

    my $Attributes = {
        'Parent'        => $self->_createOrder(),
    };
    my $MockedLineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $Attributes);
    $self->{'LineItemContainer'} = $MockedLineItemContainer;
    return $MockedLineItemContainer;
}

#----------------------------------------------------------------------------------------
sub _createPaymentTypeMethod {
    my $self = shift;

    my $Attributes = {
        'ProductType' => 'cc',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::ExternalPayment::API::Object::PaymentTypeMethod', $Attributes);
}

#----------------------------------------------------------------------------------------
sub _createOrder {
    my $self = shift;
    my $Attributes = {
        'BillingAddress' => $self->_createBillingAddress(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', $Attributes);
}

#----------------------------------------------------------------------------------------
sub _createBillingAddress {
    my $self = shift;
    my $Attributes = {
        'FullName'     => "Roger Dom\x{e8}nech",
        'Street'   => 'Diagonal 435',
        'Zipcode'      => '08036',
        'City'     => 'Barcelona',
        'EMail'    => 'r.domenech@epages.es',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', $Attributes);
}

#----------------------------------------------------------------------------------------
sub _createInvalidParams {
    my $self = shift;

    my $hParams = {
        'shopper_id' => '53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2',
        'trx_amount' =>  '1245',
        'trx_currency' => 'GBP',
        'trxuser_id' => '99999',
        'addr_city' => 'BCN',
        'addr_email' => 'r.domenech@epages.es',
        'addr_name' => "Roger Dom\x{e8}nech",
        'addr_street' => 'Diagonal 435',
        'addr_zip' => '08036',
        'client_name' => 'ePages',
        'REMOTE_ADDR' => '212.227.34.666',
        'ChangeAction' => 'HiddenTrigger',
    };

    return $hParams
}

#----------------------------------------------------------------------------------------
sub _createParams {
    my $self = shift;
    my ($IsInvalid) = @_;

    my $hParams = {
        'shopper_id' => '53DF2F8A-6A94-D04B-C88B-C0A8380AA4A2',
        'trx_amount' =>  '1245',
        'trx_currency' => 'GBP',
        'trx_paymenttyp' => 'cc',
        'trx_typ' => 'auth',
        'trxuser_id' => '99999',
        'addr_city' => 'Barcelona',
        'addr_email' => 'r.domenech@epages.es',
        'addr_name' => "Roger Dom\x{e8}nech",
        'addr_street' => 'Diagonal 435',
        'addr_zip' => '08036',
        'client_name' => 'epages',
        'REMOTE_ADDR' => '212.227.34.219',
    };

    return $hParams
}

__PACKAGE__->runScript();
1;