#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::OrderProcess
# base         Base
# state        public
# description  Description
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::OrderProcess;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Ipayment::Hooks::OrderProcess;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject;
use Test::Exception;
use Test::More;

use strict;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            Test_GetIpaymentSessionIdOk();
            Test_GetIpaymentSessionIdError();
            Test_SetSelectedTransactionType();
            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub Test_GetIpaymentSessionIdOk{
    my $SubTestName = (caller(0))[3];

    my $IsSessionStatusOK = 1;
    my $hParams = createhParams($IsSessionStatusOK);
    lives_ok(
        sub {
            DE_EPAGES::Ipayment::Hooks::OrderProcess::GetIpaymentSessionId($hParams)
        }, "$SubTestName - when createSessionAtIpaymentAndStoreIdInDatabase returns 1"
    );
}

#----------------------------------------------------------------------------------------
sub Test_GetIpaymentSessionIdError {
    my $SubTestName = (caller(0))[3];

    my $IsSessionStatusOK = 0;
    my $hParams = createhParams($IsSessionStatusOK);
    dies_ok(
        sub {
            DE_EPAGES::Ipayment::Hooks::OrderProcess::GetIpaymentSessionId($hParams)
        }, "$SubTestName - when createSessionAtIpaymentAndStoreIdInDatabase returns 0"
    );
}
#----------------------------------------------------------------------------------------
sub Test_SetSelectedTransactionType {
    my $SubTestName = (caller(0))[3];

    my $IsSessionStatusOK = 1;
    my $hParams = createhParams($IsSessionStatusOK);
    DE_EPAGES::Ipayment::Hooks::OrderProcess::SetSelectedTransactionType($hParams);
    my $SelectedTransactionType = _extractSelectedTransactionType($hParams);
    is($SelectedTransactionType, 'TestTransactionType', "$SubTestName - Selected transaction type has the expected value");
}
#----------------------------------------------------------------------------------------
sub createhParams{
    my ($IsSessionStatusOK) = @_;
    my $hParams = {
        'Servlet' => createServlet(),
        'Session' => createSession($IsSessionStatusOK)
    };
    return $hParams;
}

#----------------------------------------------------------------------------------------
sub createServlet{

    my $hAttributes = {
        'Object'     => createBasketOrder(),
        'Form'       => createForm()
    };

    my $MockedServlet = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Presentation::Servlet::Permission',$hAttributes);

    $MockedServlet->mock('object', sub{
        my $self = shift;
        return $MockedServlet->get('Object');
    });

    $MockedServlet->mock('form', sub{
        my $self = shift;
        return $MockedServlet->get('Form');
    });
    return $MockedServlet;
}

#----------------------------------------------------------------------------------------
sub createForm{
    my $Form = DE_EPAGES::Presentation::API::Form->new();
    my $MockedForm = Test::MockObject::Extends->new( $Form );
    $MockedForm->mock('executeFormError', sub{
        my $self = shift;
        my ($hErrorInfo) = @_;
        my $ErrorName = $hErrorInfo->{'Name'};
        my $ErrorReason = $hErrorInfo->{'Reason'};
        die("the input field $ErrorName is not correct ($ErrorReason).");
    });
}

#----------------------------------------------------------------------------------------
sub createSession{
    my ($IsSessionStatusOK) = @_;
    my $Session = DE_EPAGES::Ipayment::API::Session->new();
    my $MockedSession = Test::MockObject::Extends->new( $Session );
    $MockedSession->mock('createSessionAtIpaymentAndStoreIdInDatabase', sub{
        if ($IsSessionStatusOK){
            return 1;
        } else {
            return 0;
        }
    });
    return $MockedSession;
}

#----------------------------------------------------------------------------------------
sub createBasketOrder{

    my $hAttributes = {
        'LineItemContainer' => createLineItemContainer(),
        'Site' => createSite()
    };

    my $MockedBasketOrder = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAttributes);
    $MockedBasketOrder->mock('container', sub{
        my $self = shift;
        return $MockedBasketOrder->get('LineItemContainer');
    });

    $MockedBasketOrder->mock('getSite', sub{
        my $self = shift;
        return $MockedBasketOrder->get('Site');
    });

    return $MockedBasketOrder;
}

#----------------------------------------------------------------------------------------
sub createLineItemContainer{
    my $hAttributes = {
            'Payment' => createLineItemPaymentIpayment()
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);
}

#----------------------------------------------------------------------------------------
sub createSite{
    my $hAttributes = {
        'OrderProcess'     => '1',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
}

#----------------------------------------------------------------------------------------
sub createLineItemPaymentIpayment{
    my $hAttributes = {
        'SessionID'     => '',
        'instanceOf'    => 'LineItemPaymentIpayment',
        'SelectedTransactionType' => '',
        'PaymentMethod' => createPaymentMethodIpayment()
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Ipayment::API::Object::LineItemPaymentIpayment', $hAttributes);
}
#----------------------------------------------------------------------------------------
sub createPaymentMethodIpayment {
    my $hAttributes = {
        'SelectedTransactionType' => 'TestTransactionType'

    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Ipayment::API::Object::PaymentMethodIpayment', $hAttributes);
}
#----------------------------------------------------------------------------------------
sub _extractSelectedTransactionType {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrder = $Servlet->object();
    my $LineItemContainer = $BasketOrder->container();
    my $LineItemPayment = $LineItemContainer->get('Payment');
    my $SelectedTransactionType = $LineItemPayment->get('SelectedTransactionType');

    return $SelectedTransactionType;
}
#----------------------------------------------------------------------------------------
RunScript( Sub => \&Main );

1;