#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::PaymentTypeMethodsIpayment
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for Ipayment API PaymentTypeMethodsIpayment
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::PaymentTypeMethodsIpayment;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Ipayment::API::PaymentTypeMethodsIpayment;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;

use strict;

#----------------------------------------------------------------------------------------
sub test {
    my $self = shift;
    $self->test_GetPaymentsWithTransactionTypes();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_GetPaymentsWithTransactionTypes {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $PaymentMethod = $self->_PaymentMethodIpayment();
    my $PaymentTypeMethods  = DE_EPAGES::Ipayment::API::PaymentTypeMethodsIpayment->new($PaymentMethod);
    my $hPaymenMethodsTable = $PaymentTypeMethods->ExtractPaymentMethodsTable();

    my $hExpectedReturnValue = {
        'PaymentMethodTable' => [
            {'PaymentMethodID' => 123, 'TransactionTypes' => ['auth','preauth'], 'Options3dSecure' => ['1','0']},
            {'PaymentMethodID' => 654, 'TransactionTypes' => ['auth'], 'Options3dSecure' => ['1','0']},
            {'PaymentMethodID' => 2342, 'TransactionTypes' => ['repreauth'], 'Options3dSecure' => ['0']},
            {'PaymentMethodID' => 666, 'TransactionTypes' => ['preauth'], 'Options3dSecure' => ['0']},
        ]
    };
    is_deeply( $hPaymenMethodsTable, $hExpectedReturnValue, "$SubTestName - test if we get the right stucture from 'ExtractPaymentMethodsTable'");
}

#----------------------------------------------------------------------------------------
sub _PaymentMethodIpayment {
    my $self = shift;
    my $hAttributes = {
        'PaymentType' => $self->_PaymentType(),
    };
    my $MockedPaymentMethodIpayment = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Ipayment::API::Object::PaymentMethodIpayment', $hAttributes
    );
    return $MockedPaymentMethodIpayment;
}

#----------------------------------------------------------------------------------------
sub _PaymentType {
    my $self = shift;
    my $hAttributes = {
        'Class' => 'PaymentType',
        'PaymentTypeMethods'=> $self->_PaymentTypeMethods(),
    };

    my $MockedPaymentType = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Payment::API::Object::PaymentType', $hAttributes
    );

    $MockedPaymentType->mock('instanceOf', sub {
        shift;
        my ($ClassName) = @_;
        my $IsObjectOfClass = $ClassName eq $MockedPaymentType->get('Class');
        return $IsObjectOfClass;
    });

    $MockedPaymentType->mock('child',sub {
        my $MockedSelf = shift;
        return $MockedSelf->get('PaymentTypeMethods');
    });

    return $MockedPaymentType;
}

#----------------------------------------------------------------------------------------
sub _PaymentTypeMethods {
    my $self = shift;

    my $PaymentTypeMethodsGroup_CreditCard = $self->_PaymentTypeMethodsGroup( $self->_PaymentTypeMethod_CreditCard(), {'en' => 'Credit Cards'} );
    my $PaymentTypeMethodsGroup_PayPal = $self->_PaymentTypeMethodsGroup( $self->_PaymentTypeMethod_PayPal(), {'en' => 'PayPal'} );
    my $hAttributes = {
        'PaymentTypeMethodsGroup_CreditCard' => $PaymentTypeMethodsGroup_CreditCard,
        'Children' => [$PaymentTypeMethodsGroup_CreditCard, $PaymentTypeMethodsGroup_PayPal],
    };
    my $MockedPaymentTypeMethods = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalPayment::API::PaymentTypeMethods', $hAttributes
    );
    $MockedPaymentTypeMethods->mock('child',sub {
        my $MockedSelf = shift;
        my ($ChildName) = @_;
        my $ReturnValue = '';
        if ( $ChildName eq 'CreditCard' ){
            $ReturnValue = $MockedSelf->get('PaymentTypeMethodsGroup_CreditCard');
        } else {
            Error("Undefined PaymentTypeGroup: '$ChildName'");
        }
        return $ReturnValue;
    });
    $MockedPaymentTypeMethods->mock('children', sub {
        my $MockedSelf = shift;
        return $MockedSelf->get('Children');
    });

    return $MockedPaymentTypeMethods;
}

#----------------------------------------------------------------------------------------
sub _PaymentTypeMethodsGroup {
    my $self = shift;
    my ( $aPaymentTypeMethod, $Name ) = @_;
    my $hAttributes = {
        'NameOrAlias' => $Name,
        'Children' => $aPaymentTypeMethod,
    };
    my $MockedPaymentTypeMethodsGroup = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalPayment::API::Object::PaymentTypeMethodsGroup', $hAttributes
    );
    $MockedPaymentTypeMethodsGroup->mock('children', sub {
        my $MockedSelf = shift;
        return $MockedSelf->get('Children');
    });
    return $MockedPaymentTypeMethodsGroup;
}

#----------------------------------------------------------------------------------------
sub _PaymentTypeMethod_CreditCard {
    my $self = shift;
    return [
        $self->_PaymentTypeMethod(
            {'en' => 'Master-Card'},
            123, ## no critic (ProhibitMagicNumbers)
            'MasterCard',
            'cc',
            '{"TransactionTypes":["auth","preauth"]}',
            '{"Options3dSecure":["1","0"]}'),
        $self->_PaymentTypeMethod(
            {'en' => 'Visa-Card'},
            654, ## no critic (ProhibitMagicNumbers)
            'Visa',
            'cc',
            '{"TransactionTypes":["auth"]}',
            '{"Options3dSecure":["1","0"]}')
    ];
}

#----------------------------------------------------------------------------------------
sub _PaymentTypeMethod_PayPal {
    my $self = shift;
    return [
        $self->_PaymentTypeMethod(
            {'en' => 'PayPal1'},
            2342, ## no critic (ProhibitMagicNumbers)
            'PayPal',
            'paypal',
            '{"TransactionTypes":["repreauth"]}',
            '{"Options3dSecure":["0"]}'),
        $self->_PaymentTypeMethod(
            {'en' => 'PayPal1'},
            666, ## no critic (ProhibitMagicNumbers)
            'PayPal',
            'paypal',
            '{"TransactionTypes":["preauth"]}',
            '{"Options3dSecure":["0"]}')
    ];
}

#----------------------------------------------------------------------------------------
sub _PaymentTypeMethod {
    my $self = shift;
    my ($Name, $Id, $PaymentTypeKey, $ProductType, $TransactionTypes, $Options3dSecure) = @_;
    my $hAttributes = {
        'NameOrAlias' => $Name,
        'ID' => $Id,
        'PaymentTypeKey' => $PaymentTypeKey,
        'ProductType' => $ProductType,
        'TransactionTypes' => $TransactionTypes,
        'Options3dSecure'  => $Options3dSecure,
        'GUID' => '123',
    };

    my $MockedPaymentTypeMethod = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalPayment::API::Object::PaymentTypeMethod', $hAttributes
    );

    return $MockedPaymentTypeMethod;
}

__PACKAGE__->runScript();