#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::SOAP_CredentialsValidation
# state        public
#----------------------------------------------------------------------------------------
# description  Test class for DE_EPAGES::Ipayment::API::SOAP::CredentialsValidation
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::SOAP_CredentialsValidation;



use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Ipayment::API::SOAP::CompiledClient;
use DE_EPAGES::Ipayment::API::SOAP::CredentialsValidation;
use DE_EPAGES::Ipayment::t::unit::TestTools qw ( TestExistsMethod );
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject::Extends;
use Test::Exception;
use Test::More;

use feature qw{ switch };

use strict;



sub test {
    my $self = shift;
    $self->test_credentialsValidationSuccessfulCase();
    $self->test_credentialsValidationFailTraceError();
    $self->test_credentialsValidationFailError();

    done_testing();
}



sub test_credentialsValidationSuccessfulCase(){
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsInTestMode = 1;
    my $CompiledClient = _CreateCompiledClient('success');
    my $CredentialsValidation = $self->_createCredentialsValidationObject($CompiledClient, $IsInTestMode);

    my $ValidStatus = $CredentialsValidation->validateCredentials($self->_createValidhFormData($IsInTestMode));

    is($CredentialsValidation->{'ServicesURL_MockedVar'},'www.TestDummyURL.de',"$SubTestName - Test if the ServerURL was handle to the compileClient");
    my $expected_complex_structure = $self->_createRequestData();
    is_deeply($CompiledClient->{'hRequestData_MockedVar'}, $expected_complex_structure, "$SubTestName - Test if the request param reach the request client");
    my $ExpectedErrorMessage = undef;
    is($CredentialsValidation->getMessage(), $ExpectedErrorMessage,"$SubTestName - Test if the error message is empty");
    my $ExpectedValidStatus = 1;
    is($ValidStatus, $ExpectedValidStatus, "$SubTestName - Test if the credentials are valid");
}



sub test_credentialsValidationFailTraceError(){
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsInTestMode = 0;
    my $CompiledClient = _CreateCompiledClient('traceError');
    my $CredentialsValidation = $self->_createCredentialsValidationObject($CompiledClient, $IsInTestMode);
    my $ValidStatus = $CredentialsValidation->validateCredentials($self->_createValidhFormData($IsInTestMode));
    is($CredentialsValidation->{'ServicesURL_MockedVar'},'www.LiveDummyURL.de',"$SubTestName - Test if the ServerURL was handle to the compileClient");

    my $expected_complex_structure = $self->_createRequestData();
    is_deeply($CompiledClient->{'hRequestData_MockedVar'}, $expected_complex_structure, "$SubTestName - Test if the request param reach the request client");

    my $ExpectedErrorMessage = 'ErrorAtCredentialsValidation';
    is($CredentialsValidation->getMessage(), $ExpectedErrorMessage,"$SubTestName - Test if the TraceError message was create");

    my $ExpectedValidStatus = 0;
    is($ValidStatus, $ExpectedValidStatus, "$SubTestName - Test if the credentials are not valid");
}



sub test_credentialsValidationFailError(){
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $IsInTestMode = 0;
    my $CompiledClient = _CreateCompiledClient('error');
    my $CredentialsValidation = $self->_createCredentialsValidationObject($CompiledClient, $IsInTestMode);
    my $ValidStatus = $CredentialsValidation->validateCredentials($self->_createValidhFormData($IsInTestMode));
    is($CredentialsValidation->{'ServicesURL_MockedVar'},'www.LiveDummyURL.de',"$SubTestName - Test if the ServerURL was handle to the compileClient");

    my $expected_complex_structure = $self->_createRequestData();
    is_deeply($CompiledClient->{'hRequestData_MockedVar'}, $expected_complex_structure, "$SubTestName - Test if the request param reach the request client");

    my $ExpectedErrorMessage = 'ErrorAtCredentialsValidation';
    is($CredentialsValidation->getMessage(), $ExpectedErrorMessage,"$SubTestName - Test if the Error message was create");

    my $ExpectedValidStatus = 0;
    is($ValidStatus, $ExpectedValidStatus, "$SubTestName - Test if the credentials are not valid");
}



#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createCredentialsValidationObject(){
    my $self = shift;
    my ($CompiledClient, $IsInTestMode) = @_;
    my $CredentialsValidation = DE_EPAGES::Ipayment::API::SOAP::CredentialsValidation->new();

    my $MockedCredentialsValidation = Test::MockObject::Extends->new($CredentialsValidation);

    TestExistsMethod($CredentialsValidation,'_GetWebServiceUrl');
    $MockedCredentialsValidation->mock('_GetWebServiceUrl', sub { return _GetWebServiceUrl($IsInTestMode); } );

    TestExistsMethod($CredentialsValidation,'_LogPayment');
    $MockedCredentialsValidation->mock('_LogPayment', sub { return; } );

    TestExistsMethod($CredentialsValidation,'_createCompiledClient');
    $MockedCredentialsValidation->mock('_createCompiledClient', sub {
        my $self = shift;
        my ($ServiceUrl) = @_;
        $self->{'ServicesURL_MockedVar'} = $ServiceUrl;
        return $CompiledClient;
    });

    return $MockedCredentialsValidation;
}


sub _CreateTrace($){
    my ($IsError) = @_;

    my $MockedTrace = Test::MockObject->new();
    $MockedTrace->mock('response', sub { return _CreateTraceResponse($IsError); });

    return $MockedTrace;
}


sub _CreateTraceResponse($){
    my ($IsError) = @_;

    my $MockedTraceResponse = Test::MockObject->new();
    $MockedTraceResponse->mock('is_error', sub {return $IsError;});
    $MockedTraceResponse->mock('message', sub {return 'errorMessage';});
    return $MockedTraceResponse;
}



sub _CreateCompiledClient ($){
    my ($Status) = @_;
    my $CompiledClient = DE_EPAGES::Ipayment::API::SOAP::CompiledClient->new('dummyURL','DummyMethod');
    my $MockedCompiledClient = Test::MockObject::Extends->new($CompiledClient);

    TestExistsMethod($CompiledClient,'createClient');
    $MockedCompiledClient->mock('createClient', sub {
        my $self = shift;
        return sub {
            my ($hRequestData) = @_;
            $self->{'hRequestData_MockedVar'} = $hRequestData;
            return _CreateWebserviceCall($Status);
        };
    });
    return $MockedCompiledClient;
}


sub _CreateWebserviceCall($){
    my ($Status) = @_;
    given ($Status) {
        when ('success') { return ('dummyresponse',_CreateTrace(0)); }
        when ('traceError') { return ('dummyresponse',_CreateTrace(1)); }
        when ('error') {
            Error('Error: error message');
            return ('dummyresponse',_CreateTrace(1));
        }
    }
    return undef;
}



sub _createValidhFormData($){
    my $self = shift;
    my ($IsInTestMode) = @_;
    return {
          'IsInTestMode' => $IsInTestMode,
          'UserID' => 'validUserID',
          'AdminActionPassword' => 'validAdminPassword',
          'AccountID' => 'validAccountID',
          'UserPassword' => 'validUserPassword'
        };
}



sub _createRequestData(){
    return {
          'email' => 'support@ipayment.de',
          'checkPort' => 'false',
          'accountData' => {
                             'accountId' => 'validAccountID',
                             'trxpassword' => 'validUserPassword',
                             'trxuserId' => 'validUserID',
                             'adminactionpassword' => 'validAdminPassword'
                           }
        };
}



sub _GetWebServiceUrl ($) {
    my ($IsInTestMode) = @_;
    if ( $IsInTestMode) {
        return 'www.TestDummyURL.de';
    } else {
        return 'www.LiveDummyURL.de';
    }
}



__PACKAGE__->runScript();



1;