#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Tools
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::Tools;
use base DE_EPAGES::Core::API::Test;
use strict;



use Test::More;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::Tools qw ( 
    DoesFileExist
    PreparePaymentURL
);

my $TestValidationPath = "$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/Ipayment/t/unit/" . 'Tools.t';

sub TestPlan {
    test_DoesFileExistSuccess();
    test_DoesFileExistFail();
    test_DoesFileExistUndefPath();

    test_PreparePaymentURLInTestMode();
    test_PreparePaymentURLInLiveMode();

    done_testing();
}

sub test_DoesFileExistSuccess() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $DoesFileExist = DoesFileExist($TestValidationPath);
    my $ExpectedValue = 1;

    is($DoesFileExist,$ExpectedValue,"$SubTestName - Test if the file exist on the given path.");
}

sub test_DoesFileExistFail() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $DoesFileExist = DoesFileExist($TestValidationPath.'myWrongPath');
    my $ExpectedValue = 0;

    is($DoesFileExist,$ExpectedValue,"$SubTestName - Test if the file does NOT exist on the given path.");
}


sub test_DoesFileExistUndefPath() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $DoesFileExist = DoesFileExist(undef);
    my $ExpectedValue = 0;

    is($DoesFileExist,$ExpectedValue,"$SubTestName - Test with undef path.");
}


sub test_PreparePaymentURLInTestMode(){
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AccountID = 'myAccountIDTestMode';
    my $IsInTestMode = 1;
    my $URL = PreparePaymentURL($AccountID, $IsInTestMode);

    my $ExpectedValueURL = 'https://ipayment.de/merchant/myAccountIDTestMode/processor/2.0/';

    is($URL,$ExpectedValueURL,"$SubTestName - Test the URL in test mode.");
}


sub test_PreparePaymentURLInLiveMode(){
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $AccountID = 'myAccountIDLiveMode';
    my $IsInTestMode = 0;
    my $URL = PreparePaymentURL($AccountID, $IsInTestMode);

    my $ExpectedValueURL = 'https://ipayment.de/merchant/myAccountIDLiveMode/processor/2.0/';

    is($URL,$ExpectedValueURL,"$SubTestName - Test the URL in live mode.");
}

RunScript( Sub => \&TestPlan );