#========================================================================================
# package       DE_EPAGES::Ipayment::t::unit::Validator
# state        public
#----------------------------------------------------------------------------------------
# description  unit test
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::Validator;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::SOAP::Request::Validator;
use Test::More;
use Test::Exception;

my $TestValidationPath = "$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/Ipayment/t/unit/" . 'TestValidation.xsd';
sub TestPlan {

    test_DefaultConfiguration();

    test_WithUndefPath();
    test_WithInvalidXSDFile();
    test_WithValidXSDFileAndValidHash();
    test_WithValidXSDFileAndUndefHash();
    test_WithValidXSDFileAndValidHashNodeButNotValidDataType();
    test_WithValidXSDFileAndInvalidHashNode();
    test_WithValidXSDFileAndInvalidAttributeName();

    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_DefaultConfiguration {
    my $SubTestName = (caller(0))[3];
    my $Validator = DE_EPAGES::Ipayment::API::SOAP::Request::Validator->new();
    my $hTestRequest = {
        'one' => 'eins'
    };
    ok($Validator->isValidHash($hTestRequest),"$SubTestName - test default validator which will always return 1");
}

#----------------------------------------------------------------------------------------
sub test_WithUndefPath {
    my $SubTestName = (caller(0))[3];

    my $hTestRequest = {
        'one' => 'eins'
    };
    my $ValidatorXsd = DE_EPAGES::Ipayment::API::SOAP::Request::Validator->new();
    my $Path = undef;
    my $Status = $ValidatorXsd->setXsd($Path);

    is ($Status, 0, "$SubTestName - if the Filepath is a string but it is not a valid path");

    throws_ok { $ValidatorXsd->isValidHash($hTestRequest) } (
        qr/XSD-file could not be loaded/,
        "$SubTestName - Error that XSD file could not be loaded"
        );
}

#----------------------------------------------------------------------------------------
sub test_WithInvalidXSDFile {
    my $SubTestName = (caller(0))[3];

    my $hTestRequest = {
        'one' => 'eins'
    };
    my $ValidatorXsd = DE_EPAGES::Ipayment::API::SOAP::Request::Validator->new( );
    my $Path = 'not a File but a string';
    my $Status =$ValidatorXsd->setXsd($Path);

    is ($Status, 0, "$SubTestName - if the Filepath is a string but it is not a valid path");

    throws_ok { $ValidatorXsd->isValidHash($hTestRequest) } (
        qr/XSD-file could not be loaded/,
        "$SubTestName - Error that XSD file could not be loaded"
        );
}

#----------------------------------------------------------------------------------------
sub test_WithValidXSDFileAndValidHash {
    my $SubTestName = (caller(0))[3];

    my $ValidatorXsd = DE_EPAGES::Ipayment::API::SOAP::Request::Validator->new( );
    $ValidatorXsd->setXsd($TestValidationPath);
    my $hTestHash = {'TestAttribute' =>{
            'TestElement_Int' => 123456,
            'TestElement_String' => 'jajaaj',
        }};

    my $IsValidHash = $ValidatorXsd->isValidHash($hTestHash);
    ok($IsValidHash,"$SubTestName - tests if all is fine");
}

#----------------------------------------------------------------------------------------
sub test_WithValidXSDFileAndUndefHash {
    my $SubTestName = (caller(0))[3];

    my $ValidatorXsd = DE_EPAGES::Ipayment::API::SOAP::Request::Validator->new( );
    $ValidatorXsd->setXsd($TestValidationPath);
    my $hUndefHash = '';

    my $IsValidHash = $ValidatorXsd->isValidHash($hUndefHash);
    is($IsValidHash, 0, "$SubTestName - Tests if undef hash is invalid");
}

#----------------------------------------------------------------------------------------
sub test_WithValidXSDFileAndValidHashNodeButNotValidDataType {
    my $SubTestName = (caller(0))[3];

    my $ValidatorXsd = DE_EPAGES::Ipayment::API::SOAP::Request::Validator->new( );
    $ValidatorXsd->setXsd($TestValidationPath);
    my $hTestHash = {'TestAttribute' =>{
            'TestElement_Int' => 'bliblub',
            'TestElement_String' => 123456,
        }};
    my $IsValidHash = $ValidatorXsd->isValidHash($hTestHash);
    is($IsValidHash, 0,"$SubTestName - tests if the type of the data is not Valid");
}

#----------------------------------------------------------------------------------------
sub test_WithValidXSDFileAndInvalidHashNode {
    my $SubTestName = (caller(0))[3];

    my $ValidatorXsd = DE_EPAGES::Ipayment::API::SOAP::Request::Validator->new( );
    $ValidatorXsd->setXsd($TestValidationPath);
    my $hTestHash = {'TestAttribute' =>{
            'WrongNodeName' => 'bliblub',
        }};
    my $IsValidHash = $ValidatorXsd->isValidHash($hTestHash);
    is($IsValidHash, 0,"$SubTestName - tests if the element name of the data is not Valid");
}

#----------------------------------------------------------------------------------------
sub test_WithValidXSDFileAndInvalidAttributeName {
    my $SubTestName = (caller(0))[3];

    my $ValidatorXsd = DE_EPAGES::Ipayment::API::SOAP::Request::Validator->new( );
    $ValidatorXsd->setXsd($TestValidationPath);
    my $hTestHash = {'WrongAttributeName' =>'wrong'};
    my $IsValidHash = $ValidatorXsd->isValidHash($hTestHash);
    is($IsValidHash, 0,"$SubTestName - tests if the attribute name of the data is not Valid");
}

RunScript( Sub => \&TestPlan );